@@ -1,5 +1,5 @@
-/* Fault handler information.  Linux/m68k version.
-   Copyright (C) 2002  Bruno Haible <bruno@clisp.org>
+/* Fault handler information.  Linux/m68k version when it supports POSIX.
+   Copyright (C) 2002, 2009  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,10 +15,14 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#include <asm/sigcontext.h>
-#include "fault-linux-m68k.c"
+#include "fault-posix-ucontext.h"
 
-#define SIGSEGV_FAULT_HANDLER_ARGLIST  int sig, int code, struct sigcontext *scp
-#define SIGSEGV_FAULT_ADDRESS  (unsigned long) get_fault_addr (scp)
-#define SIGSEGV_FAULT_CONTEXT  scp
-#define SIGSEGV_FAULT_STACKPOINTER  scp->sc_usp
+/* See glibc/sysdeps/unix/sysv/linux/m68k/sys/ucontext.h
+   and the definition of GET_STACK in
+   glibc/sysdeps/unix/sysv/linux/m68k/sigcontextinfo.h.
+   Note that the 'mcontext_t' defined in
+   glibc/sysdeps/unix/sysv/linux/m68k/sys/ucontext.h
+   and the 'struct sigcontext' defined in <asm/sigcontext.h>
+   are quite different types.  */
+
+#define SIGSEGV_FAULT_STACKPOINTER  ((ucontext_t *) ucp)->uc_mcontext.gregs[R_SP]
