@@ -1,5 +1,5 @@
 /* Fault handler information.  MacOSX version.
-   Copyright (C) 1993-1999, 2002-2003, 2007-2008, 2016  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 1993-1999, 2002-2003, 2007-2008, 2016, 2018, 2020  Bruno Haible <bruno@clisp.org>
    Copyright (C) 2003  Paolo Bonzini <bonzini@gnu.org>
 
    This program is free software; you can redistribute it and/or modify
@@ -73,7 +73,7 @@
 
 /* This is not defined in any header, although documented.  */
 
-/* http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/exc_server.html says:
+/* https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/exc_server.html says:
    The exc_server function is the MIG generated server handling function
    to handle messages from the kernel relating to the occurrence of an
    exception in a thread. Such messages are delivered to the exception port
@@ -92,7 +92,7 @@ extern boolean_t
                    mach_msg_header_t *reply_msg);
 
 
-/* http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/catch_exception_raise.html
+/* https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/catch_exception_raise.html
    These functions are defined in this file, and called by exc_server.
    FIXME: What needs to be done when this code is put into a shared library? */
 kern_return_t
@@ -208,13 +208,20 @@ catch_exception_raise (mach_port_t exception_port,
   uintptr_t sp;
 
 #ifdef DEBUG_EXCEPTION_HANDLING
+  /* Notes:
+     - The exception types are listed in xnu-VERSION/osfmk/mach/exception_types.h.
+       Value 0x1 is EXC_BAD_ACCESS.
+     - The first code is the exception code, listed in
+         xnu-VERSION/osfmk/mach/i386/exception.h
+         xnu-VERSION/osfmk/mach/arm/exception.h
+     - The second code is the address.  */
   fprintf (stderr, "Exception: 0x%x Code: 0x%x 0x%x in catch....\n",
            exception,
            code_count > 0 ? code[0] : -1,
            code_count > 1 ? code[1] : -1);
 #endif
 
-  /* See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_get_state.html.  */
+  /* See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_get_state.html.  */
 #ifdef SIGSEGV_EXC_STATE_TYPE
   state_count = SIGSEGV_EXC_STATE_COUNT;
   if (thread_get_state (thread, SIGSEGV_EXC_STATE_FLAVOR,
@@ -264,8 +271,10 @@ catch_exception_raise (mach_port_t exception_port,
 #else
         stk_extra_stack + 256;
 #endif
-#if defined __x86_64__ || defined __i386__
+#if defined __aarch64__ || defined __x86_64__ || defined __i386__
       new_safe_esp &= -16; /* align */
+#endif
+#if defined __x86_64__ || defined __i386__
       new_safe_esp -= sizeof (void *); /* make room for (unused) return address slot */
 #endif
       SIGSEGV_STACK_POINTER (thread_state) = new_safe_esp;
@@ -293,7 +302,7 @@ catch_exception_raise (mach_port_t exception_port,
       SIGSEGV_PROGRAM_COUNTER (thread_state) = (uintptr_t) terminating_handler;
     }
 
-  /* See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_set_state.html.  */
+  /* See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_set_state.html.  */
   if (thread_set_state (thread, SIGSEGV_THREAD_STATE_FLAVOR,
                         (void *) &thread_state, state_count)
       != KERN_SUCCESS)
@@ -311,7 +320,7 @@ catch_exception_raise (mach_port_t exception_port,
 static void *
 mach_exception_thread (void *arg)
 {
-  /* See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/mach_thread_self.html.  */
+  /* See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/mach_thread_self.html.  */
   our_exception_thread = mach_thread_self ();
 
   for (;;)
@@ -402,7 +411,7 @@ mach_initialize ()
       != KERN_SUCCESS)
     return -1;
 
-  /* See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/mach_port_insert_right.html.  */
+  /* See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/mach_port_insert_right.html.  */
   if (mach_port_insert_right (self, our_exception_port, our_exception_port,
                               MACH_MSG_TYPE_MAKE_SEND)
       != KERN_SUCCESS)
@@ -426,7 +435,7 @@ mach_initialize ()
      for a particular thread.  This has the effect that when our exception
      port gets the message, the thread specific exception port has already
      been asked, and we don't need to bother about it.
-     See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/task_set_exception_ports.html.  */
+     See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/task_set_exception_ports.html.  */
   if (task_set_exception_ports (self, mask, our_exception_port,
                                 EXCEPTION_DEFAULT, MACHINE_THREAD_STATE)
       != KERN_SUCCESS)
@@ -477,7 +486,7 @@ sigsegv_leave_handler (void (*continuation) (void*, void*, void*),
           return 0;
         }
 
-      /* See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_get_state.html.  */
+      /* See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_get_state.html.  */
       state_count = SIGSEGV_THREAD_STATE_COUNT;
       if (thread_get_state (thread, SIGSEGV_THREAD_STATE_FLAVOR,
                             (void *) &thread_state, &state_count)
@@ -490,7 +499,7 @@ sigsegv_leave_handler (void (*continuation) (void*, void*, void*),
           return 0;
         }
 
-#if defined __ppc64__ || defined __ppc__ || defined __x86_64__
+#if defined __aarch64__ || defined __ppc64__ || defined __ppc__ || defined __x86_64__
       /* Store arguments in registers.  */
       SIGSEGV_INTEGER_ARGUMENT_1 (thread_state) = (uintptr_t) cont_arg1;
       SIGSEGV_INTEGER_ARGUMENT_2 (thread_state) = (uintptr_t) cont_arg2;
@@ -521,7 +530,7 @@ sigsegv_leave_handler (void (*continuation) (void*, void*, void*),
       /* Point program counter to continuation to be executed.  */
       SIGSEGV_PROGRAM_COUNTER (thread_state) = (uintptr_t) continuation;
 
-      /* See http://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_set_state.html.  */
+      /* See https://web.mit.edu/darwin/src/modules/xnu/osfmk/man/thread_set_state.html.  */
       if (thread_set_state (thread, SIGSEGV_THREAD_STATE_FLAVOR,
                             (void *) &thread_state, state_count)
           != KERN_SUCCESS)
